/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2015                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2015                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/
#ifndef __ASSIGNMENT_EVALUATION_H
#define __ASSIGNMENT_EVALUATION_H 0

#include "CubeNullaryEvaluation.h"
#include "CubePLMemoryManager.h"

namespace cube
{
class AssignmentEvaluation : public NullaryEvaluation
{
protected:

    CubePLMemoryManager* memory;
    std::string          variable_name;
    MemoryAdress         variable;
    KindOfVariable       kind;
    GeneralEvaluation*   value;
    GeneralEvaluation*   index;


public:
    AssignmentEvaluation( std::string          _v,
                          GeneralEvaluation*   _index,
                          GeneralEvaluation*   _val,
                          CubePLMemoryManager* _manager
                          ) : NullaryEvaluation(), memory( _manager ), variable_name( _v ), variable( memory->register_variable( _v ) ), value( _val ), index( _index )
    {
        kind = memory->kind_of_variable( variable_name );
    };
    ~AssignmentEvaluation();

    inline
    virtual
    void
    setRowSize( size_t size )
    {
        value->setRowSize( size );
        index->setRowSize( size );
        GeneralEvaluation::setRowSize( size );
    }


    virtual
    double
    eval();
    virtual
    double
    eval( Cnode*             cnode,
          CalculationFlavour cf,
          Sysres*            sr,
          CalculationFlavour tf  );

    virtual
    double
    eval( Cnode*             cnode,
          CalculationFlavour cf );

    virtual
    double*
    eval_row( Cnode*             _cnode,
              CalculationFlavour _cf );

    virtual
    double
    eval( double arg1,
          double arg2 );


    virtual
    void
    print()
    {
        std::cout << "${" << variable_name << "}[";
        index->print();
        std::cout << "] = ";
        value->print();
        std::cout << ";" << std::endl;
    };
};
};

#endif
